#############################################################################
#############################################################################
#
# latex.tcl, version 3.0:  macros, menu, and key bindings for LaTeX users
#
#############################################################################
#
# version 1.1 and 1.2 (11/10/92) by Richard T. Austin (austin@eecs.umich.edu)
# version 2.0--2.3 and 3.0 (7/95) by Tom Scavo (trscavo@syr.edu)
#
# If you make improvements to this file, please share them!
#
#############################################################################
#############################################################################

#############################################################################
#
# Autoload procedures
#
#############################################################################

proc dummyTeX {} {}
proc latexMenu {} {}

#############################################################################
#
# Flags and variables
#
#############################################################################

set true 1
set false 0

# Flags:
newModeVar TeX deleteEnvironmentNoisily $true 1
newModeVar TeX deleteObjectNoisily $true 1
# newModeVar TeX expertMode $false 1
newModeVar TeX promptNoisily $true 1
newModeVar TeX runTeXInBackground $false 1
newModeVar TeX searchNoisily $true 1
newModeVar TeX smartDots $true 1
newModeVar TeX smartQuotes $true 1
newModeVar TeX smartScripts $true 1
newModeVar TeX useBoxMacro $true 1
# newModeVar TeX useBracketsInDisplaymath $false 1
newModeVar TeX useDollarSigns $false 1
newModeVar TeX useStatusBar $true 1
newModeVar TeX wordWrap $true 1

# Variables:
# newModeVar TeX TeXInputs {} 0
newModeVar TeX citeCommands {cite nocite} 0
newModeVar TeX fillColumn {70} 0
# Bug---will not select arguments with embedded braces:
newModeVar TeX funcExpr {\\(sub)*section(\[.*\]|\*)?({[^{}]*})?} 0
newModeVar TeX funcExprAlt {\\section(\[.*\]|\*)?({[^{}]*})?} 0 
# newModeVar TeX funcExpr {\\((sub)*section|chapter)(\[.*\]|\*)?({[^{}]*})?} 0
# newModeVar TeX funcExprAlt {\\(section|chapter)(\[.*\]|\*)?({[^{}]*})?} 0 
newModeVar TeX leftFillColumn {0} 0
newModeVar TeX prefixString {% } 0
newModeVar TeX refCommands {ref pageref} 0
# Will not find  \<space> :
newModeVar TeX wordBreak {([a-zA-Z-0-9]+|\\(\\\*?|[^A-Za-z\t\r* \]|[A-Za-z]+\*?))} 0
newModeVar TeX wordBreakPreface {([^a-zA-Z-0-9\]|.\\)} 0
newModeVar TeX wrapBreak {([a-zA-Z-0-9]+|\\(\\\*?|[^A-Za-z\t\r* \]|[A-Za-z]+\*?))} 0
newModeVar TeX wrapBreakPreface {([^a-zA-Z-0-9\]|.\\)} 0

# The proc 'shadowUseBoxMacro' is executed automatically whenever 
# the flag 'useBoxMacro' is altered.
trace variable TeXmodeVars(useBoxMacro) w shadowUseBoxMacro
proc shadowUseBoxMacro {{name1 {}} {name2 {}} {op w}} {
	global TeXmodeVars
	if {$TeXmodeVars(useBoxMacro)} then {
		# Undim 'boxMacroName':
		if {[catch {set TeXmodeVars((boxMacroName)}]} then {
			newModeVar TeX boxMacroName {includegraphics} 0
		} else {
			newModeVar TeX boxMacroName $TeXmodeVars((boxMacroName) 0
			unset TeXmodeVars((boxMacroName)
		}
	} else {
		# Dim 'boxMacroName':
		if {[catch {set TeXmodeVars(boxMacroName)}]} then {
			newModeVar TeX {(boxMacroName} {includegraphics} 0
		} else {
			newModeVar TeX {(boxMacroName} $TeXmodeVars(boxMacroName) 0
			unset TeXmodeVars(boxMacroName)
		}
	}
}
shadowUseBoxMacro

#############################################################################
#
# Colorization
#
#############################################################################

set primaryLaTeXColor "blue"

# The proc 'shadowCiteRefCommands' is executed whenever the variables 
# TeXmodeVars(citeCommands) or TeXmodeVars(refCommands) are changed.
trace variable TeXmodeVars(citeCommands) w shadowCiteRefCommands
trace variable TeXmodeVars(refCommands) w shadowCiteRefCommands
proc shadowCiteRefCommands {{name1 {}} {name2 {}} {op w}} {
	global TeXmodeVars primaryLaTeXColor
	set LaTeXClickWords {}
	foreach word $TeXmodeVars(citeCommands) { append LaTeXClickWords " \\\\$word" }
	foreach word $TeXmodeVars(refCommands) { append LaTeXClickWords " \\\\$word" }
	# color and underline command-double-clickable LaTeX commands:
	regModeKeywords -a -u -k $primaryLaTeXColor TeX $LaTeXClickWords
}

# The proc 'shadowBoxMacroName' is executed whenever the variable 
# TeXmodeVars(boxMacroName) is changed.
trace variable TeXmodeVars(boxMacroName) w shadowBoxMacroName
proc shadowBoxMacroName {{name1 {}} {name2 {}} {op w}} {
	global TeXmodeVars primaryLaTeXColor
	set LaTeXClickWords {}
	foreach word $TeXmodeVars(boxMacroName) { append LaTeXClickWords " \\\\$word" }
	# color and underline command-double-clickable LaTeX commands:
	regModeKeywords -a -u -k $primaryLaTeXColor TeX $LaTeXClickWords
}

proc colorLaTeXClickCommands {} {
	global primaryLaTeXColor
	set LaTeXClickWords { \\documentclass \\usepackage }
	append LaTeXClickWords { \\input \\include }
	append LaTeXClickWords { \\bibliography \\bibliographystyle }
	append LaTeXClickWords { \\LoadClass \\RequirePackage }
	# color and underline command-double-clickable LaTeX commands:
	regModeKeywords -a -u -k $primaryLaTeXColor TeX $LaTeXClickWords
	shadowCiteRefCommands
	shadowBoxMacroName
}

proc colorLaTeXCommands {} {
	global primaryLaTeXColor
	set LaTeXKeyWords {}
	# Color comments (red), braces and dollar signs (green), and 
	# all LaTeX commands ($primaryLaTeXColor):
	regModeKeywords -i "\}" -i "\{" -i "\$" -I green \
	                -e {%} -m {\\} -c red -k $primaryLaTeXColor TeX $LaTeXKeyWords
	# Color sectioning commands (magenta):
	regModeKeywords -a -k magenta TeX \
		{ \\part \\chapter \\section \\subsection \\subsubsection \\paragraph }
}

colorLaTeXCommands
colorLaTeXClickCommands

#############################################################################
#
# LaTeX core routines
#
#############################################################################

source "$HOME:Tcl:SystemCode:latexEngine.tcl"

#############################################################################
#
# LaTeX macros
#
#############################################################################

source "$HOME:Tcl:SystemCode:latexMacros.tcl"

#############################################################################
#
# LaTeX menu definition
#
#############################################################################

source "$HOME:Tcl:SystemCode:latexMenu.tcl"
buildLaTeXMenu

#############################################################################
#
# LaTeX key bindings
#
#############################################################################

source "$HOME:Tcl:SystemCode:latexKeys.tcl"
bindLaTeXKeys

#############################################################################
#
# Smart quotes and dots
#
#############################################################################

source "$HOME:Tcl:SystemCode:latexSmart.tcl"

#############################################################################
#
# Patches and bug fixes
#
#############################################################################

catch {source "$HOME:Tcl:SystemCode:latexPatches.tcl"}

#############################################################################
#############################################################################

message "LaTeX initialization complete"

